/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SRC_VISION_ABSTRACTSOCKET_H_
#define SRC_VISION_ABSTRACTSOCKET_H_

extern "C" {
#include <netdb.h>
}

#include <ostream>

#include "SocketState.h"

class AbstractSocket {
  /* atributes */
 protected:
  mutable int error_;  //!< Member variable "error_"
  int file_descriptor_;  //!< Member variable "file_descriptor_"
  SocketState state_;  //!< Member variable "state_"
  /* methods */
 public:
  /** Default constructor */
  AbstractSocket();
  /** Default destructor */
  virtual ~AbstractSocket();
  /** Copy constructor
   * \param other Object to copy from
   */
  explicit AbstractSocket(const AbstractSocket& other);
  /** Constructor that allows to define all parameters.
   * \param file_descriptor File descriptor of socket.
   * \param state State of this socket.
   */
  AbstractSocket(const int file_descriptor, const SocketState state);
  friend std::ostream& operator<<(std::ostream& out, AbstractSocket&
      abstractSocket);
  /** Assignment operator
   * \param other Object to assign from
   * \return A reference to this
   */
  AbstractSocket& operator=(const AbstractSocket& rhs);
  int get_error() const;
  bool close();
  bool isOpen() const;
  bool isBounded() const;
  bool isClosed() const;
  bool isConnected() const;
  bool isListening() const;

 protected:
  virtual bool open() = 0;
  //virtual void printOn(std::ostream& out) const = 0;
  // get sockaddr, IPv4 or IPv6:
  void* get_in_addr(struct sockaddr *sa);
};

#endif  // SRC_VISION_ABSTRACTSOCKET_H_
