/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BLOB_H_
#define BLOB_H_

#include <opencv2/opencv.hpp>
#include <iostream>
#include "UAVision.h"
#include "ScanLines.h"
#include "RLE.h"

namespace uav{
/** Structure for the definition of the information associated to a blob. */

class BlobInfo
{
public:
	int center, upperLeft, downRight;
	double area;
	double widhtHeightRel;
	int nRle;
	int dist;
	int coloredPixels;
	double solidity;

	BlobInfo(RLE rle, int idx)
	{
		center = rle.rlData[idx].center;
		upperLeft = rle.rlData[idx].start;
		downRight = rle.rlData[idx].end;
		nRle = 1;
		area = 1.0 * rle.rlData[idx].lengthColor * rle.s.getStep();
		widhtHeightRel = area;
		dist = 0;
		coloredPixels = rle.rlData[idx].lengthColor;
		solidity = 1.0;
	}
};

class Blob
{

public:

	std::vector<BlobInfo> blobs;
	//RLE rle;

	Blob();

	inline cv::Point getPointXYFromInteger(int idx, RLE &rle)
	{
		cv::Point temp;

		temp.x = idx%rle.s.image.cols;
		temp.y = idx/rle.s.image.cols;

		return temp;
	}

	inline int getIntFromXY(cv::Point pt, RLE &rle)
	{
		int temp;

		temp = pt.y * rle.s.image.cols + pt.x;

		return temp;
	}

	void createBlobs(RLE &rle_, std::vector<int> &threshold, cv::Point &robotCenter);
	void updateBlob(BlobInfo &b, RLE &rle, int idx, cv::Point &robotCenter);
	int findBlob(RLE &rle_, std::vector<int> &threshold, int idx);
	double distance(cv::Point p1, cv::Point p2);
	void sort(unsigned sortMethod, RLE &rle);
	void draw(cv::Scalar color, cv::Mat &destination);

};
}
#endif
