/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CAMERACALIB_H_
#define CAMERACALIB_H_

#include <opencv2/opencv.hpp>
#include <iostream>
#include "UAVision.h"
#include "CameraSettings.h"

namespace uav{

#define MSVTHRESHOLD 0.1
#define UVTHRESHOLD 5

class CameraCalib
{

public:

	int hist_size;
	cv::Mat image;
	std::vector <int> histValue;

	CameraCalib(int size, cv::Mat &imagePassed);

	~CameraCalib(){}

	void calcHistograms(cv::Mat &mask);

	float calcMean(cv::Mat &hist);

	float calcMSV(std::vector<int> &hist);

	float calcACM(cv::Mat &hist);

	cv::Scalar calcUV(cv::Rect &rect);

	cv::Scalar calcRGB(cv::Rect &rect);

	bool CalibrateCamera(CameraSettings *camSettings, ParameterRange *parameterRange,
			cv::Mat &mask, cv::Rect &rWhite, bool firstTimeCalib, bool verbose,
			std::vector<bool> &paramChanged, bool wbCalib = true);

};
}
#endif
