/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef CAMERAFIREWIRE_H_
#define CAMERAFIREWIRE_H_

#include <libraw1394/raw1394.h>
#include <dc1394/dc1394.h>
#include "UAVision.h"
#include "Camera.h"
#include "CameraSettingsFirewire.h"

namespace uav{

class CameraSettingsFirewire;

class CameraFirewire: public Camera
{
public:

	dc1394camera_t *camera;


	CameraFirewire(){}

	~CameraFirewire(){}


	/**
	 * Method for start sending images.
	 */
	void startCamera(CameraSettings *camSettings );

	/**
	 * Method for stopping the camera and finishing the connection.
	 */
	void shutDownCamera( void );

	/**
	 * Method for initializing the camera.
	 */
	int initCamera(CameraSettings *camSettings);

	/**
	 * Method for retrieving the camera settings.
	 * @param *camSettings - camera settings to be retrieved.
	 */
	void getParameters(CameraSettings *camSettings);


	/**
	 * Method for retrieving the limits of the value of a camera parameter.
	 * @param parameter - camera parameter to be retrieved.
	 * @param parameterRange - values of the limits of the parameter to be filled.
	 */
	void getParametersRanges( ParameterRange parameterRange[] );


	/**
	 * Pure virtual method for reading a frame from the camera.
	 * @param &imageOpenCV - frame read.
	 */
	int readFrame( cv::Mat &imageOpenCV );

	/**
	 * Method for setting the camera in full auto-mode.
	 * @param *camSettings - camera settings that are to be set in automode.
	 */
	void setAutoManualMode(CameraSettings *camSettings );

	void setAutoMode(CameraSettings *camSettings );


	/**
	 * Method for setting the camera in manual mode.
	 * @param *camSettings - camera settings to be set in manual mode.
	 */
	void setManualMode(CameraSettings *camSettings );

	/**
	 * Method for printing the camera settings.
	 */
	void printParameters();


	/**
	 * Method for printing just one camera parameter.
	 * @param parameter - camera parameter whose value should be printed.
	 */
	void printParameter(unsigned int parameter);

	/**
	 * Method for initializing the camera.
	 */
	int init(CameraSettings *camSettings);


	/**
	 * Method for stop sending images.
	 */
	void stopCamera(CameraSettings *camSettings );


	/**
	 * Method for setting camera parameters.
	 * @param *camSettings - camera parameters to be set.
	 */
	void setParameters(CameraSettings *camSettings);

	/**
	 * Pure virtual method for setting the value of one camera parameter.
	 * @param parameter - camera parameter to be set.
	 * @param value - new value of the parameter to be set.
	 */
	void setParameter(unsigned int parameter, unsigned value);

	/**
	 * Method for retrieving the value of one camera parameter.
	 * @param parameter - camera parameter to be retrieved.
	 * @param *value - value of the parameter to be retrieved.
	 */
	void getParameter(unsigned int parameter, double *value);



	/**
	 * Method for retrieving the limits of the value of a camera parameter.
	 * @param parameter - camera parameter to be retrieved.
	 * @param parameterRange - values of the limits of the parameter to be filled.
	 */
	void getParameterRanges(unsigned int parameter,ParameterRange parameterRange);

	void rewind(){}

private:

	int setFps(unsigned int fps);


	//Mat& operator>> ();
};
}

#endif
