/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "CameraFromFile.h"

namespace uav{

CameraFromFile::CameraFromFile( const std::string fileName_, CameraSettings *camSettings)
{
	fileName = fileName_;

	if(camSettings->getCameraSetting(UAV_VIDMODE) == UAV_GRAY)
		image = cv::Mat(cv::Size(camSettings->getCameraSetting(UAV_NCOLS),
				camSettings->getCameraSetting(UAV_NROWS)),CV_8UC1);
	else
		if(camSettings->getCameraSetting(UAV_VIDMODE) == UAV_RGB8)

			image = cv::Mat(cv::Size(camSettings->getCameraSetting(UAV_NCOLS),
					camSettings->getCameraSetting(UAV_NROWS)),CV_8UC3);

	file.open(fileName.c_str(), std::fstream::in |  std::fstream::binary);

	if( file.fail() )
	{
		std::cout<< "Something failed on loading the file" << std::endl;
	}
}

void CameraFromFile::rewind()
{
	file.close();
	file.open(fileName.c_str(), std::fstream::in |  std::fstream::binary);
}

int CameraFromFile::readFrame(cv::Mat &imageToReturn)
{
	file.read((char*)image.ptr(), image.rows * image.cols * image.channels());
	//cv::cvtColor(image, image, CV_BGR2);
	/*std::vector<cv::Mat> canais;
	canais.resize(3);
	cv::split(image, canais);
	cv::Mat r, g, b;
	canais[0].copyTo(b);
	canais[1].copyTo(r);
	canais[2].copyTo(g);
	r.copyTo(canais[0]);
	g.copyTo(canais[1]);
	b.copyTo(canais[2]);
	cv::merge(canais, image);*/
	image.copyTo(imageToReturn);

	/*
	std::vector<int> compression_params;
	compression_params.push_back(CV_IMWRITE_PNG_COMPRESSION);
	compression_params.push_back(9);
	cv::imwrite("raw.png", image, compression_params);
	cv::Mat color;
	cv::cvtColor(image, color, CV_BayerBG2BGR);
	cv::imwrite("rgb.png", color, compression_params);
	*/
	if(!file)
	{
		std::cerr << "ERROR: only " << file.gcount() << " bytes could be read from video file\n";
		return -1;
	}
	else
		return 0;
}

int CameraFromFile::init(CameraSettings *camSettings)
{
	return 0;
}

int CameraFromFile::initCamera(CameraSettings *camSettings)
{
	return 0;
}

void CameraFromFile::startCamera(CameraSettings *camSettings)
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::stopCamera(CameraSettings *camSettings)
{

	std::cout<<"Method not supported!" << std::endl;

}

void CameraFromFile::shutDownCamera()
{

	std::cout<<"Method not supported" <<std::endl;

}

void CameraFromFile::getParameter(unsigned parameter, double *value)
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::getParameterRanges(unsigned parameter, ParameterRange parameterRange)
{

	std::cout<<"Method not supported for this type of camera!"<<std::endl;
}

void CameraFromFile::getParameters(CameraSettings *camSettings)
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::getParametersRanges(ParameterRange* parameterRange)
{
	std::cout<<"Method not supported for this type of camera!" <<std::endl;
}

void CameraFromFile::printParameter(unsigned parameter)
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::printParameters()
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::setAutoMode(CameraSettings *camSettings)
{
	std::cout<<"Method not supported for this type of camera!"<<std::endl;
}

void CameraFromFile::setManualMode(CameraSettings *camSettings)
{
	std::cout<<"Method not supported for this type of camera!"<<std::endl;
}

void CameraFromFile::setParameter(unsigned parameter, unsigned value)
{
	std::cout<<"Method not supported" << std::endl;
}

void CameraFromFile::setParameters(CameraSettings *camSettings)
{
	setParameter(UAV_FPS, camSettings->getCameraSetting(UAV_FPS));
}

}
