/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LUT_H_
#define LUT_H_

#include <ctime>
#include <iostream>
#include <fstream>
#include <vector>
#include <boost/filesystem/operations.hpp>
#include <opencv2/opencv.hpp>
#include "Config.h"

namespace uav{

#define LUT_MAX_Y		256
#define LUT_MAX_U		256
#define LUT_MAX_V		256
#define LUT_MEMORY_SIZE	( LUT_MAX_Y * LUT_MAX_U * LUT_MAX_V )

class Lut
{
	public:
		unsigned char *info;
		std::vector<int> validPixels;

		Lut();
		Lut(Config& config);

		~Lut();

		unsigned char &operator[](const unsigned index)
		{
			return info[index];
		}

		unsigned char &operator[](const unsigned index) const
		{
			return info[index];
		}

		void createValidPixels(cv::Mat &mask, int mode);
		void createLUT(int mode, ColorRange* cr);
		void convertImageToIndex(cv::Mat &original, cv::Mat &index, int imageMode);
		void init(ColorRange* cr);
		void saveToCache(std::string cacheFilename);
		void loadFromCache(std::string cacheFilename);
};
}
#endif
