/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "PID_vision.h"

namespace uav{


PID::PID( float kp , float ki , float kd , float minOut, float maxOut , float maxInt ) 
{
	this->kp		= kp;
	this->ki		= ki; 
	this->kd		= kd;
	this->minOut	= minOut;
	this->maxOut	= maxOut;
	this->maxInt	= maxInt;
	
	this->lastError	= 0.0;
	this->integral	= 0.0;
	this->firstTime	= true;
}

PID::PID( const PID& pid ) 
{
	this->kp		= pid.kp;
	this->ki		= pid.ki; 
	this->kd		= pid.kd;
	this->minOut	= pid.minOut;
	this->maxOut	= pid.maxOut;
	this->maxInt	= pid.maxInt;
	
	this->lastError	= pid.lastError;
	this->integral	= pid.integral;
	this->firstTime	= true;
}

float PID::compensate(float value, float error )
{
	integral += ki * error;

	if(integral > maxInt)
		integral = maxInt;
	else
	if(integral < -maxInt)
		integral = -maxInt;

	float out = 0.0;

	if(firstTime)
	{
		integral = 0;
		out = value + kp * error;
		firstTime = false;
	}
	else
		out = value + (kp * error + kd * (error - lastError) + integral);

	if(out > maxOut )
		out = maxOut;
	
	if(out < minOut)
		out = minOut;

	lastError = error;
	
	return out;
}

void PID::reset()
{
	firstTime = true;
}

void PID::display() 
{
	printf("  p(%.2f) i(%.2f) d(%.2f) maxOut(%.2f) maxInt(%.2f)\n", kp, ki, kd, maxOut, maxInt);	
}
}
