/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SCANLINES_H_
#define SCANLINES_H_

#include <opencv2/opencv.hpp>
#include <iostream>
#include "UAVision.h"
#include <math.h>

namespace uav{
class ScanLines
{

public:

	cv::Point start; /** Starting point of the scan line. */
	cv::Point end; /** Ending point of the scan line. */
	unsigned stepX; /** Horizontal lines sparsity */
	unsigned stepY; /** Vertical lines sparsity */
	unsigned inRadius; /** Inner radius for radial scan lines.*/
	unsigned outRadius; /** Outer radius for radial scan lines.*/
	unsigned type; /** Type of the scanline (Radial, Circular, Horizontal or Vertical */
	cv::Point center; /** The center of the radial and circular scanlines */
	cv::Mat image; /** Reference image used to construct the Scanlines */
	std::vector <std::vector <int> > scanlines; /** Data container */

	ScanLines();

	/**
	 * Constructor for the creation of horizontal/vertical scan lines.
	 */
	ScanLines(cv::Mat &image_, unsigned scanType, cv::Point start_, cv::Point end_, unsigned step_x, unsigned step_y);

	/**
	 * Constructor for the creation of radial and circular scan lines.
	 */
	ScanLines (cv::Mat &image_, unsigned scanType, cv::Point center, unsigned nSensors,
			unsigned inRad, unsigned outRad, int radiusStep = 1);

	/**
	 * Destructor.
	 */
	~ScanLines(){}

	int signum(int d);

	inline std::vector<int>& operator[](int idx) {return scanlines[idx];}

	inline const std::vector<int>& operator[](int idx) const {return scanlines[idx];}

	inline std::vector<int> getLine(int idx) {return scanlines[idx];}

	inline int getPoint(int i, int j) {return scanlines[i][j];}

	inline cv::Point getPointXYFromInteger(int idx)
	{
		cv::Point temp;

		temp.x = idx%image.cols;
		temp.y = idx/image.cols;

		return temp;
	}

	inline cv::Point getPointXY(int i, int j)
	{
		cv::Point temp;
		temp.x = scanlines[i][j]%image.cols;
		temp.y = scanlines[i][j]/image.cols;

		return temp;
	}

	unsigned getStep();

	void draw(cv::Mat &destination, cv::Scalar color);
};
}
#endif
