/*
 * Copyright (C) 2014 Alina Trifan and António Neves
 * mailto: alina.trifan@ua.pt -- an@ua.pt 
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "UAVision.h"
#include "stdio.h"

namespace uav{

void drawLine(int pt1, int pt2, cv::Scalar color, cv::Mat &img)
{
	cv::line(img, cv::Point(pt1 % img.cols, pt1 / img.cols), cv::Point(pt2 % img.cols, pt2 / img.cols), color, 2);
}

void drawCircle(int pt, int radius, cv::Scalar color, cv::Mat &img)
{
	cv::circle(img, cv::Point(pt % img.cols, pt / img.cols), radius, color, 4);
}

/*
void InitMask(vector<int> &validPixels, int mode, unsigned char *mask, unsigned width, unsigned height)
{
	int offset, mb, imb;

	for(unsigned i = 0 ; i < width * height ; i++)
		if(mask[i] > 0)
		{
			switch(mode)
			{
				case RGB8:
					validPixels.push_back(i); // index
					validPixels.push_back(i * 3); // R
					validPixels.push_back(i * 3 + 1); // G
					validPixels.push_back(i * 3 + 2); // B
					break;

				case YUV422: // UYVY
					offset = 2 * i + 1;
					validPixels.push_back(i); // index
					validPixels.push_back(offset); // Y
					if(i % 2 == 0)
					{
						validPixels.push_back(offset - 1); // U
						validPixels.push_back(offset + 1); // V
					}
					else
					{
						validPixels.push_back(offset - 3); // U
						validPixels.push_back(offset - 1); // V
					}

					break;

				case YUV411: // UYYVYY
					mb = i / 4;
					imb = i % 4;
					validPixels.push_back(i); // index
					validPixels.push_back(mb * 6 + imb + imb / 2 + 1); // Y
					validPixels.push_back(mb * 6); // U
					validPixels.push_back(mb * 6 + 3); // V

				case GRAY:

					int row = i / width;
					int col = i % width;
					int r = 0, g = 0, b = 0;

					if(row % 2 == 0) // linhas pares
					{
						if(col % 2 == 0) // colunas pares
						{
							r = i;
							g = i + 1;
							b = i + width + 1;
						}
						else
						{
							g = i;
							r = i - 1;
							b = i + width;
						}
					}
					else // linhas impares
					{
						if(col % 2 == 0) // colunas pares
						{
							g = i ;
							r = i - width;
							b = i + 1;
						}
						else // colunas impares
						{
							g = i - 1;
							r = i - width - 1;
							b = i;
						}
					}

					validPixels.push_back(i); // index
					validPixels.push_back(r); // R
					validPixels.push_back(g); // G
					validPixels.push_back(b); // B

			}

		}

}
 */

void Yuv2Rgb2(float y, float u, float v, float *r, float *g, float *b)
{
	int c, d, e;
	int modo = 2;

	switch(modo)
	{
	case 0:
		*b = (y + 1.772  * (u-128));
		*g = (y - 0.34414 * (u-128) - 0.71414 * (v-128));
		*r = (y			 + 1.402  * (v-128));
		break;

	case 1:
		*b = (1.164 * (y - 16)                     + 2.018 * (u - 128));
		*g = (1.164 * (y - 16) - 0.813 * (v - 128) - 0.391 * (u - 128));
		*r = (1.164 * (y - 16) + 1.596 * (v - 128));
		break;

	case 2:
		*b = 1.164*(y - 16) + 2.018*(u-128);
		*g = 1.164*(y - 16) - 0.813*(u-128) - 0.391*(v-128);
		*r = 1.164*(y - 16) + 1.596*(v-128);
		break;

	case 3:
		// Wikipedia
		c = (int)(y - 16);
		d = (int)(u - 128);
		e = (int)(v - 128);
		*r = ( 298 * c           + 409 * e + 128) >> 8;
		*g = ( 298 * c - 100 * d - 208 * e + 128) >> 8;
		*b = ( 298 * c + 516 * d           + 128) >> 8;
		break;

	case 4:
		// Prof. Bernardo
		*r = y + 1.14 * v;
		*g = y - 0.395 * u - 0.581 * v;
		*b = y + 2.032 * u;
		break;

	case 5:
		// yuvtoppm
		u = u - 128;
		v = v - 128;
		y = y - 16;
		//if (y < 0) y = 0;


		*r = 104635 * v;
		*g = -25690 * u + -53294 * v;
		*b = 132278 * u;

		y *= 76310;

		*r = *r + y;
		*g = *g + y;
		*b = *b + y;
		break;
	}

	if(*r < 0) *r = 0;
	if(*g < 0) *g = 0;
	if(*b < 0) *b = 0;
	if(*r > 255) *r = 255;
	if(*g > 255) *g = 255;
	if(*b > 255) *b = 255;
}

void Yuv2Rgb(int y, int u, int v, int *r, int *g, int *b)
{
	*r = ( 298 * (y - 16) + 409 * (v - 128) + 128) >> 8;
	*g = ( 298 * (y - 16) - 100 * (u - 128) - 208 * (v - 128) + 128) >> 8;
	*b = ( 298 * (y - 16) + 516 * (u - 128) + 128) >> 8;

	if(*r < 0) *r = 0;
	if(*g < 0) *g = 0;
	if(*b < 0) *b = 0;
	if(*r > 255) *r = 255;
	if(*g > 255) *g = 255;
	if(*b > 255) *b = 255;
}

void Rgb2Yuv(int r, int g, int b, int *y, int *u, int *v)
{
	*y = 0.299*r + 0.587*g + 0.114*b;
	*u = (r-*y)*0.713 +127;
	*v = (b-*y)*0.564 + 127;


	if(*y < 0) *y = 0;
	if(*u < 0) *u = 0;
	if(*v < 0) *v = 0;
	if(*y > 255) *y = 255;
	if(*u > 255) *u = 255;
	if(*v > 255) *v = 255;


}

void Rgb2Hsv( int r, int g, int b, int *hout, int *sout, int *vout )
{
	float min, max, delta;
	float h, s, v;

	min = min3( r, g, b );
	max = max3( r, g, b );
	v = max;				// v
	delta = max - min;
	if( max != 0 )
		s = delta / max * 100;		// s
	else
	{
		// r = g = b = 0		// s = 0, v is undefined
		s = 0;
		h = -1;
		return;
	}
	if( r == max )
		h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == max )
		h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		h = 4 + ( r - g ) / delta;	// between magenta & cyan
	h *= 60;				// degrees
	if( h < 0 )
		h += 360;

	*hout = (int)h;
	*sout = (int)s;
	*vout = (int)v;
}

void Rgb2Hsv2( float r, float g, float b, float *h, float *s, float *v)
{
	float min, max, delta;
	min = min3( r, g, b );
	max = max3( r, g, b );
	*v = max;				// v
	delta = max - min;
	if( max != 0 )
		*s = delta / max * 100;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*s = 0;
		*h = -1;
		return;
	}
	if( r == max )
		*h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == max )
		*h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		*h = 4 + ( r - g ) / delta;	// between magenta & cyan
	*h *= 60;				// degrees
	if( *h < 0 )
		*h += 360;

}

void distRelation (std::vector<int> &data, cv::Mat &image, bool full)
{
	double p3 = -2.3922e-07;
	double p2 = 3.3582e-04;
	double p1 = -1.7131e-01;
	double p0 = 3.7821e+01;

	//double m = -0.12525;
	//double b = 54.40597;
	int tmp;

	for(int d = 0 ; d < 1001 ; d++)
	{
		tmp = (int)(p3 * pow(d, 3) + p2 * pow(d, 2) + p1 * d + p0);
		if(tmp < 6) tmp = 6;
		data.push_back(tmp);
	}

	if(full) return;

	for(int d = 0 ; d < 40 && d < data.size(); d++)
	{
		data[d] = (int)(data[d] * 0.8);
	}
}

void distRelationGoalie(std::vector<int> &data, cv::Mat &image)
{
	for(int i = 0 ; i < image.cols * image.rows ; i++)
		data.push_back(50);
}

}
